local addon = MissionMinder;
addon.skipfights = {};
local skipfights = addon.skipfights;
local options = addon.options;
local DEBUG = MissionMinder.debug;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

local SKIP_ANIMATION_CONTROL = {
	[1] = options.SkipFightAnimations,	-- line between encounters
	[2] = options.SkipFightAnimations,	-- check that models are loaded
	[3] = options.SkipFightAnimations,	-- model fight
	[4] = options.SkipFightAnimations,	-- impact sound when follower hits
	[5] = options.SkipFightAnimations,	-- X over portrait
	[6] = options.SkipFightAnimations,	-- evaluate whether to do next encounter or move on
	[7] = nil,				-- reward panel
	[8] = nil,				-- explode the lock if mission successful
	[9] = nil,				-- show all the mission followers
	[10] = nil				-- follower xp
};


-- Skip any fight animations and sound as defind in SKIP_ANIMATION_CONTROL
local function skipAnimations(...)
	if ( options.SkipFightAnimations() ) then
		local self, elapsed = ...

		local skipControl = SKIP_ANIMATION_CONTROL[self.animIndex];
		while ( skipControl and skipControl() ) do
			self.animTimeLeft = 0;
			skipControl = SKIP_ANIMATION_CONTROL[self.animIndex + 1];

			if(skipControl and skipControl()) then
				-- only increase animation index if the next one is ALSO skipped. This is necessary because the original
				--OnUpdate function will increase animation index for us becaue animTimeLeft <= 0
				--The result is that leaving this loop, self.animIndex == 6, but will be increased to 7 for us.
				self.animIndex = self.animIndex + 1;
			end
		end
	end

	skipfights.GarrisonMissionComplete_OnUpdate(...);
end


-- if SkipFightAnimations, hide enemy unit frames from completion frame
local function hideEncounterFrame(...)
	if ( options.SkipFightAnimations() ) then
		GarrisonMissionFrame.MissionComplete.Stage.EncountersFrame:Hide();
	end
end


-- Show mission success/failure message
local function showSuccessFailureMessage_ptr(self, missionID, canComplete, succeeded)
	if ( self.currentMission and self.currentMission.missionID == missionID ) then
		if( options.SkipFightAnimations() and self.currentMission ) then
			local self = GarrisonMissionFrame.MissionComplete;
			self.BonusRewards.Saturated:Show();
			self.BonusRewards.Saturated.FadeIn:Play();
			if ( self.currentMission.succeeded ) then
				self.ChanceFrame.ResultText:SetText(GARRISON_MISSION_SUCCESS);
				self.ChanceFrame.ResultText:SetTextColor(0.1, 1, 0.1);
				self.ChanceFrame.ResultAnim:Play();
				self.BonusRewards.ChestModel:SetAnimation(0, 0);
				PlaySound("UI_Garrison_CommandTable_MissionSuccess_Stinger");
			else
				self.ChanceFrame.ResultText:SetText(GARRISON_MISSION_FAILED);
				self.ChanceFrame.ResultText:SetTextColor(1, 0.1, 0.1);
				self.ChanceFrame.ResultAnim:Play();
				self.NextMissionButton:Enable();
				PlaySound("UI_Garrison_Mission_Complete_MissionFail_Stinger");
			end
		end
	end
end
local function showSuccessFailureMessage(self, missionID, canComplete, succeeded)
	if ( self.currentMission and self.currentMission.missionID == missionID ) then
		if( options.SkipFightAnimations() ) then
			local self = GarrisonMissionFrame.MissionComplete.ChanceFrame;
			self.ChanceText:SetAlpha(0)
			if (succeeded) then
				self.SuccessAnim:Play();
				self.SuccessText:SetAlpha(1);
				self.FailureText:SetAlpha(0);
				PlaySound("UI_Garrison_CommandTable_MissionSuccess_Stinger");
			else
				self.FailureAnim:Play();
				self.SuccessText:SetAlpha(0);
				self.FailureText:SetAlpha(1);
				PlaySound("UI_Garrison_Mission_Complete_MissionFail_Stinger");
			end
		end
	end
end


function skipfights.initialize()
	-- Skip fight animations
	-- INSECURE HOOK - Requires pre-hook to override the local animation control
	skipfights.GarrisonMissionComplete_OnUpdate = GarrisonMissionComplete_OnUpdate;
	GarrisonMissionComplete_OnUpdate = skipAnimations;
	if(GarrisonMissionFrame.MissionComplete:GetScript("OnUpdate")) then
		GarrisonMissionFrame.MissionComplete:SetScript("OnUpdate", GarrisonMissionComplete_OnUpdate);
	end

	-- Hide encounter frame (enemy unit frames)
	hooksecurefunc("GarrisonMissionComplete_Initialize", hideEncounterFrame);

	-- Show the success/failure message
	if ( MissionMinder.ptr ) then
		hooksecurefunc("GarrisonMissionComplete_OnMissionCompleteResponse", showSuccessFailureMessage_ptr);
	else
		hooksecurefunc("GarrisonMissionComplete_OnMissionCompleteResponse", showSuccessFailureMessage);
	end
end
